#!/usr/bin/perl
###############################################################################
##
## Written 2005/2006 by Oliver Schneider (Assarbad) - http://assarbad.net
##
## This self-contained script will create an NT driver project based on the
## DDKBUILD command interpreter script.
##
## No warranties, use at your own risk - the script is PUBLIC domain.
##
## WARNING: This script overwrites any files that have the same name as the
## project and end on .c, .h, .vcproj which exist in the target directory!
##
## The binary Windows console program contained in the overlay of the perl
## script is available under BSDL from my website: http://assarbad.net as
## makeguid.rar (includes source, of course).
##
## Granted, it's not nice but it works ;) ...
##
## [Version of 2006-01-17]
## - Added ${prefix}, fixed some minor issues and completed the project file.
## [Version of 2006-01-05]
## - Initial release
##
###############################################################################
use Fcntl;

print "Project generator for VC++ 2003 project files of NT driver projects\n(projects based on DDKBUILD by OSR!)\nNo warranties, use at your own risk - the script is PUBLIC domain.\n\n";
# create_proj.pl <project name> <path for the project> [solution name]
#
# If you provide a solution name (the third parameter), which is optional
# the script will put the driver project in a subdirectory of the directory
# given, named \"driver\" and put a solution containing the driver project
# in the directory given on the command line.
$syntaxhelp = "
create_proj.pl <project name> <path for the project>

Valid characters for the project name are *only* letters, underscore and
digits. The first character of the project name has to be a letter! This is
important, otherwise the script will refuse to run.

##############################################################################
## WARNING: This script overwrites any files that have the same name as the ##
## project and end on .c, .h, .vcproj which exist in the target directory!  ##
##############################################################################
";
my ($projectname, $path, $solutionname) = @ARGV;
# Check for the project name
unless($projectname =~ /^[a-z][a-z0-9_]+$/gis)
{
  die "Error: No or no valid project NAME given.\n$syntaxhelp";
}
# Check for trailing backspace
unless($path =~ /[\/\\]$/) { $path .= '/'; }
# Normalize path ...
$path =~ s/\//\\/gism;
# Define the project path
my $projpath = $path;
# if($solutionname ne '')
# {
#   $projpath .= "driver\\";
# }
# Check for the path
unless(-d $projpath)
{
  qx(md "$projpath");
  unless(-d $projpath)
  {
    die "Error: No project PATH given or not a directory ($path).\n$syntaxhelp";
  }
}
# Normalize path ...
$path =~ s/\\/\//gism;
$projpath =~ s/\\/\//gism;
# Some output
print "Creating project $projectname in directory '$path' ...\n";
# Prepare some variables
my $tempfile = "";
# Get current time for the date and year "variables"
my ($tsec, $tmin, $thour, $tmday, $tmonth,
    $tyear, $twday, $tyday, $dst) = localtime(time);
# Normalize the result
$tmonth++;
$tyear +=1900;
# Cache it ...
my $longdate = sprintf('%04d-%02d-%02d', $tyear, $tmonth, $tmday);
# Get the contents of the data section (this file after __END__)
my @fconts = <DATA>;
# Initialize some variables use lateron
my %filesdata = ();
my @binfiles = ();
my @filestodelete = ();
my $makeguid = '';
my $prefix = uc($projectname);
$prefix =~ s/_//gsm;

print "Extracting overlay of this script ...\n";
# Unpack all (TEXT) files
foreach $i (@fconts)
{
  # Replace certain variables
  $i =~ s/\${project_name}/$projectname/gsm;
#  $i =~ s/\${solution_name}/$solutionname/gsm;
  $i =~ s/\${date}/$longdate/gsm;
  $i =~ s/\${year}/$tyear/gsm;
  $i =~ s/\${prefix}/$prefix/gsm;
  # Watch out for separator between files attached to this module
  if($i =~ /^------------------------------\[(.+?)\]------------------------------/gism)
  {
    my $newtempfile = $1;
    my $pushfile = '';
    if($newtempfile =~ /^\?/i) {$newtempfile =~ s/^\?//gism; $pushfile = $newtempfile}
    if(uc($tempfile) ne uc($newtempfile))
    {
      $tempfile = $newtempfile;
      if($pushfile ne '') {push(@binfiles, $pushfile);}
      $filesdata{$tempfile} = '';
    }
  }
  else
  {
    $filesdata{$tempfile} .= $i;
  }
}

print "Now writing extracted binary file(s).\n";
# Now convert all binary files ... (FIRST STEP)
foreach $file (@binfiles)
{
  my $uudecoded = unpack("u", $filesdata{$file});
  my $tmpname = $file;
  $file =~ s/\.tmp$//i;
  sysopen(FILE, $projpath.$file, O_WRONLY | O_CREAT);
  syswrite(FILE, $uudecoded);
  close(FILE);
  delete $filesdata{$tmpname}; # Remove the textual file
  push(@filestodelete, $projpath.$file);
  if($file =~ /makeguid/i)
  {
    $makeguid = $projpath.$file;
  }
}

my $projguid = qx($makeguid);
$projguid =~ s/\n//gism;
$projguid = uc($projguid);

print "Now writing extracted textual file(s).\n";
# Finally write the files and replace the makeguidXX variables by guids
foreach $key (sort keys %filesdata)
{
  my ($data, $filepath, $filetype) = ($filesdata{$key}, $projpath.$key, 'project ');
#   if($key =~ /\.sln$/i)
#   {
#     if(uc($projpath) ne uc($path))
#     {
#       $filepath = $path.$key;
#       $filetype = 'solution '
#     }
#     else
#     {
#       next;
#     }
#   }
  open(FILE, '>'.$filepath) || die "Could not create ".$filetype."file: ".$key;
  for(my $i = 0; $i < 100; $i++)
  {
    my $regex = sprintf('%02d', $i);
    if($data =~ /\${makeguid$regex}/gism)
    {
      my $guid = qx($makeguid);
      $guid =~ s/\n//gism;
      $guid = uc($guid);
      $data =~ s/\${makeguid$regex}/$guid/gism;
    }
  }
  $data =~ s/\${proj_guid}/$projguid/gism;
  print FILE $data;
  close(FILE);
}

# Delete the files on the delete list
print "Removing obsolete temporary files ...\n";
foreach(@filestodelete)
{
  unlink($_);
}
print "Finished - make sure to review the SOURCES file in the target directory!\n";
__END__
------------------------------[${project_name}.c]------------------------------
///////////////////////////////////////////////////////////////////////////////
///
/// Copyright (c) ${year} - <company name here>
///
/// Original filename: ${project_name}.c
/// Project          : ${project_name}
/// Date of creation : ${date}
/// Author(s)        : <author name(s)>
///
/// Purpose          : <description>
///
/// Revisions:
///  0000 [${date}] Initial revision.
///
///////////////////////////////////////////////////////////////////////////////

#include <ntddk.h>
#include <string.h>

#include "${project_name}.h"

PDRIVER_OBJECT pdoGlobalDrvObj = 0;

NTSTATUS ${prefix}_DispatchCreateClose(
	IN PDEVICE_OBJECT		DeviceObject,
	IN PIRP					Irp
	)
{
	Irp->IoStatus.Status = STATUS_SUCCESS;
	Irp->IoStatus.Information = 0;
	IoCompleteRequest(Irp, IO_NO_INCREMENT);
	return Irp->IoStatus.Status;
}

NTSTATUS ${prefix}_DispatchDeviceControl(
	IN PDEVICE_OBJECT		DeviceObject,
	IN PIRP					Irp
	)
{
	PIO_STACK_LOCATION irpSp = IoGetCurrentIrpStackLocation(Irp);

	switch(irpSp->Parameters.DeviceIoControl.IoControlCode)
	{
	default:
		Irp->IoStatus.Status = STATUS_INVALID_DEVICE_REQUEST;
		Irp->IoStatus.Information = 0;
		break;
	}

	IoCompleteRequest(Irp, IO_NO_INCREMENT);
	return Irp->IoStatus.Status;
}

VOID ${prefix}_DriverUnload(
	IN PDRIVER_OBJECT		DriverObject 
	)
{
	PDEVICE_OBJECT pdoNextDeviceObj = pdoGlobalDrvObj->DeviceObject;
	IoDeleteSymbolicLink(&usSymlinkName);

	// Delete all the device objects
	while(pdoNextDeviceObj)
	{
		PDEVICE_OBJECT pdoThisDeviceObj = pdoNextDeviceObj;
		pdoNextDeviceObj = pdoThisDeviceObj->NextDevice;
		IoDeleteDevice(pdoThisDeviceObj);
	}
}

NTSTATUS DriverEntry(
	IN OUT PDRIVER_OBJECT   DriverObject,
	IN PUNICODE_STRING      RegistryPath
	)
{
	PDEVICE_OBJECT pdoDeviceObj = 0;
	NTSTATUS status = STATUS_UNSUCCESSFUL;
	pdoGlobalDrvObj = DriverObject;

	// Create the device object.
	if(!NT_SUCCESS(status = IoCreateDevice(
		DriverObject,
		0,
		&usDeviceName,
		FILE_DEVICE_UNKNOWN,
		FILE_DEVICE_SECURE_OPEN,
		FALSE,
		&pdoDeviceObj
		)))
	{
		// Bail out (implicitly forces the driver to unload).
		return status;
	};

	// Now create the respective symbolic link object
	if(!NT_SUCCESS(status = IoCreateSymbolicLink(
		&usSymlinkName,
		&usDeviceName
		)))
	{
		IoDeleteDevice(pdoDeviceObj);
		return status;
	}

	DriverObject->MajorFunction[IRP_MJ_CREATE] =
	DriverObject->MajorFunction[IRP_MJ_CLOSE] = ${prefix}_DispatchCreateClose;
	DriverObject->MajorFunction[IRP_MJ_DEVICE_CONTROL] = ${prefix}_DispatchDeviceControl;
	DriverObject->DriverUnload = ${prefix}_DriverUnload;

	return STATUS_SUCCESS;
}

------------------------------[${project_name}.h]------------------------------
///////////////////////////////////////////////////////////////////////////////
///
/// Copyright (c) ${year} - <company name here>
///
/// Original filename: ${project_name}.h
/// Project          : ${project_name}
/// Date of creation : <see ${project_name}.c>
/// Author(s)        : <see ${project_name}.c>
///
/// Purpose          : <see ${project_name}.c>
///
/// Revisions:         <see ${project_name}.c>
///
///////////////////////////////////////////////////////////////////////////////
#pragma once

#include "msg_${project_name}.h"
#include "drvcommon.h"
#include "drvversion.h"

#define DEVICE_NAME			"\\Device\\${prefix}_DeviceName"
#define SYMLINK_NAME		"\\DosDevices\\${prefix}_DeviceName"
PRESET_UNICODE_STRING(usDeviceName, DEVICE_NAME);
PRESET_UNICODE_STRING(usSymlinkName, SYMLINK_NAME);

------------------------------[sources]------------------------------
TARGETNAME=${project_name}
TARGETPATH=obj
TARGETTYPE=DRIVER
# Use additional include file
NTTARGETFILES=makefile.inc
# Create browse info
#BROWSER_INFO

SOURCES=${project_name}.c \
        msg_${project_name}.mc \
        drvversion.rc
------------------------------[makefile]------------------------------
#
# DO NOT EDIT THIS FILE!!!  Edit .\sources. if you want to add a new source
# file to this component.  This file merely indirects to the real make file
# that is shared by all the driver components of the Windows NT DDK
#

!INCLUDE $(NTMAKEENV)\makefile.def
------------------------------[makefile.inc]------------------------------
# Just additional makefile actions

------------------------------[msg_${project_name}.mc]------------------------------
;///////////////////////////////////////////////////////////////////////////////
;///
;/// Copyright (c) ${year} - <company name here>
;///
;/// Original filename: ${project_name}.h
;/// Project          : ${project_name}
;/// Date of creation : <see ${project_name}.c>
;/// Author(s)        : <see ${project_name}.c>
;///
;/// Purpose          : Message table for the driver.
;///
;/// Revisions:         <see ${project_name}.c>
;///
;///////////////////////////////////////////////////////////////////////////////
;
;#pragma once

MessageIdTypedef=NTSTATUS

SeverityNames=(Success=0x0:STATUS_SEVERITY_SUCCESS
               Informational=0x1:STATUS_SEVERITY_INFORMATIONAL
               Warning=0x2:STATUS_SEVERITY_WARNING
               Error=0x3:STATUS_SEVERITY_ERROR
              )

FacilityNames=(System=0x0)

MessageId=0x0001 Facility=System Severity=Success SymbolicName=${prefix}_SUCCESS
Language=English
Successfully did my work
.

MessageId=0x0001 Facility=System Severity=Error SymbolicName=${prefix}_ERROR
Language=English
Error performing action
.

------------------------------[${project_name}.vcproj]------------------------------
<?xml version="1.0" encoding="Windows-1252"?>
<VisualStudioProject
	ProjectType="Visual C++"
	Version="7.10"
	Name="${project_name} (Driver)"
	ProjectGUID="${proj_guid}"
	Keyword="MakeFileProj">
	<Platforms>
		<Platform
			Name="Win32"/>
	</Platforms>
	<Configurations>
		<Configuration
			Name="Checked Build (WNET)|Win32"
			IntermediateDirectory=".\"
			ConfigurationType="0">
			<Tool
				Name="VCNMakeTool"
				BuildCommandLine="ddkbuild -WNET checked ."
				ReBuildCommandLine="ddkbuild -WNET checked . -cZ"/>
		</Configuration>
		<Configuration
			Name="Free Build (WNET)|Win32"
			IntermediateDirectory=".\"
			ConfigurationType="0">
			<Tool
				Name="VCNMakeTool"
				BuildCommandLine="ddkbuild -WNET free ."
				ReBuildCommandLine="ddkbuild -WNET free . -cZ"/>
		</Configuration>
		<Configuration
			Name="Checked Build (WXP)|Win32"
			IntermediateDirectory=".\"
			ConfigurationType="0">
			<Tool
				Name="VCNMakeTool"
				BuildCommandLine="ddkbuild -WNETXP checked ."
				ReBuildCommandLine="ddkbuild -WNETXP checked . -cZ"/>
		</Configuration>
		<Configuration
			Name="Free Build (WXP)|Win32"
			IntermediateDirectory=".\"
			ConfigurationType="0">
			<Tool
				Name="VCNMakeTool"
				BuildCommandLine="ddkbuild -WNETXP free ."
				ReBuildCommandLine="ddkbuild -WNETXP free . -cZ"/>
		</Configuration>
		<Configuration
			Name="Checked Build (W2K)|Win32"
			IntermediateDirectory=".\"
			ConfigurationType="0">
			<Tool
				Name="VCNMakeTool"
				BuildCommandLine="ddkbuild -WNET2K checked ."
				ReBuildCommandLine="ddkbuild -WNET2K checked . -cZ"/>
		</Configuration>
		<Configuration
			Name="Free Build (W2K)|Win32"
			IntermediateDirectory=".\"
			ConfigurationType="0">
			<Tool
				Name="VCNMakeTool"
				BuildCommandLine="ddkbuild -WNET2K free ."
				ReBuildCommandLine="ddkbuild -WNET2K free . -cZ"/>
		</Configuration>
	</Configurations>
	<References>
	</References>
	<Files>
		<Filter
			Name="Source Files"
			Filter="cpp;c;cxx;def;odl;idl;hpj;bat;asm;asmx;mc">
			<File
				RelativePath=".\${project_name}.c">
			</File>
			<File
				RelativePath=".\msg_${project_name}.mc">
			</File>
		</Filter>
		<Filter
			Name="Header Files"
			Filter="h;hpp;hxx;hm;inl;inc;xsd">
			<File
				RelativePath=".\drvcommon.h">
			</File>
			<File
				RelativePath=".\drvversion.h">
			</File>
			<File
				RelativePath=".\${project_name}.h">
			</File>
			<File
				RelativePath=".\msg_${project_name}.h">
			</File>
		</Filter>
		<Filter
			Name="Resource Files"
			Filter="rc;ico;cur;bmp;dlg;rc2;rct;bin;rgs;gif;jpg;jpeg;jpe;resx">
		</Filter>
		<Filter
			Name="Special Files"
			Filter="*">
			<File
				RelativePath=".\ddkpostbld.cmd">
			</File>
			<File
				RelativePath=".\ddkprebld.cmd">
			</File>
			<File
				RelativePath=".\makefile">
			</File>
			<File
				RelativePath=".\makefile.inc">
			</File>
			<File
				RelativePath=".\sources">
			</File>
		</Filter>
	</Files>
	<Globals>
	</Globals>
</VisualStudioProject>
------------------------------[drvcommon.h]------------------------------
///////////////////////////////////////////////////////////////////////////////
///
/// Useful macros for driver writers.
///
/// PUBLIC DOMAIN
///
///////////////////////////////////////////////////////////////////////////////

#pragma once

#define _ANSISTRING(text) #text
#define ANSISTRING(text) _ANSISTRING(text)

#define _WIDESTRING(text) L##text
#define WIDESTRING(text) _WIDESTRING(text)

#define PRESET_UNICODE_STRING(symbol, buffer) \
        UNICODE_STRING symbol = \
            { \
            sizeof(WIDESTRING(buffer)) - sizeof(WCHAR), \
            sizeof(WIDESTRING(buffer)), \
            WIDESTRING(buffer) \
            };

#define CREATE_XVER(maj,min,build) maj ## , ## min ## , 0, ## build
#define CREATE_FVER(maj,min,build) maj ## . ## min ## .0. ## build
#define CREATE_PVER(maj,min,build) maj ## . ## min
------------------------------[drvversion.rc]------------------------------
///////////////////////////////////////////////////////////////////////////////
///
/// The version information resource. Depends on "drvversion.h" header file.
///
/// PUBLIC DOMAIN
///
///////////////////////////////////////////////////////////////////////////////

#include <winver.h>
#include <winnt.h>

#ifdef RC_INVOKED
LANGUAGE LANG_NEUTRAL, SUBLANG_NEUTRAL
#include "drvcommon.h"
#include "drvversion.h"

#ifdef DBG
#define ACTUAL_FILEFLAGS VS_FF_DEBUG
#else
#define ACTUAL_FILEFLAGS 0
#endif

VS_VERSION_INFO VERSIONINFO
 FILEVERSION CREATE_XVER(DRV_MAJVER, DRV_MINVER, DRV_BUILD)
 PRODUCTVERSION CREATE_XVER(PRD_MAJVER, PRD_MINVER, 0)
 FILEFLAGSMASK VS_FFI_FILEFLAGSMASK
 FILEFLAGS ACTUAL_FILEFLAGS
 FILEOS VOS_NT
 FILETYPE VFT_DRV
 FILESUBTYPE VFT2_DRV_SYSTEM
BEGIN
    BLOCK "StringFileInfo"
    BEGIN
        BLOCK "000004b0"
        BEGIN
#ifdef TEXT_AUTHOR
            VALUE "Author", ANSISTRING(TEXT_AUTHOR)
#endif
#ifdef TEXT_COMMENTS
            VALUE "Comments", ANSISTRING(TEXT_COMMENTS)
#endif
            VALUE "CompanyName", ANSISTRING(TEXT_COMPANY)
            VALUE "FileDescription", ANSISTRING(TEXT_FILEDESC)
            VALUE "FileVersion", ANSISTRING(CREATE_FVER(DRV_MAJVER, DRV_MINVER, DRV_BUILD))
            VALUE "InternalName", ANSISTRING(TEXT_INTERNALNAME)
            VALUE "LegalCopyright", ANSISTRING(TEXT_COPYRIGHT)
            VALUE "OriginalFilename", ANSISTRING(TEXT_MODULE)
            VALUE "ProductName", ANSISTRING(TEXT_PRODUCTNAME)
            VALUE "ProductVersion", ANSISTRING(CREATE_PVER(PRD_MAJVER, PRD_MINVER, PRD_BUILD))
#ifdef TEXT_SPECIALBUILD
            VALUE "SpecialBuild", ANSISTRING(TEXT_SPECIALBUILD)
#endif
            VALUE "Website", ANSISTRING(TEXT_WEBSITE)
        END
    END
    BLOCK "VarFileInfo"
    BEGIN
        VALUE "Translation", 0x0, 1200
    END
END

#endif // RC_INVOKED
------------------------------[drvversion.h]------------------------------
///////////////////////////////////////////////////////////////////////////////
///
/// Defines for the version information in the resource file
///
/// PUBLIC DOMAIN
///
///////////////////////////////////////////////////////////////////////////////

#pragma once

// ---------------------------------------------------------------------------
// Several defines have to be given before including this file. These are:
// ---------------------------------------------------------------------------
#define TEXT_AUTHOR            [your name here] // author (optional value)
#define PRD_MAJVER             1 // major prodduct version
#define PRD_MINVER             0 // minor product version
#define PRD_BUILD              0 // build number for product
#define DRV_MAJVER             1 // major driver version
#define DRV_MINVER             0 // minor driver version
#define DRV_BUILD              0 // build number for driver
#define DRV_YEAR               ${year} // current year
#define TEXT_WEBSITE           http:/##/www.yoursite.com // website
#define TEXT_PRODUCTNAME       Supercool driver-based tool // product's name
#define TEXT_FILEDESC          The driver for the supercool driver-based tool // component description
#define TEXT_COMPANY           [your company here] // company
#define TEXT_MODULE            ${project_name} // module name
#define TEXT_COPYRIGHT         Copyright \xA9 DRV_YEAR TEXT_COMPANY // copyright information
// #define TEXT_SPECIALBUILD      // optional comment for special builds
#define TEXT_INTERNALNAME      ${project_name}.sys // copyright information
// #define TEXT_COMMENTS          // optional comments
// ---------------------------------------------------------------------------
// ... well, that's it. Pretty self-explanatory ;)
// ---------------------------------------------------------------------------
------------------------------[ddkprebld.cmd]------------------------------
@echo off
:: Perform any pre-build steps
------------------------------[ddkpostbld.cmd]------------------------------
@echo off
:: Perform post-build steps
------------------------------[?makeguid.exe.tmp]------------------------------
M35J0``,````$````__\``+@`````````0```````````````````````````
M````````````````````T`````X?N@X`M`G-(;@!3,TA5&AI<R!P<F]G<F%M
M(&-A;FYO="!B92!R=6X@:6X@1$]3(&UO9&4N#0T*)`````````#M;R7'J0Y+
ME*D.2Y2I#DN4QA%/E*L.2Y2I#DJ4K`Y+E"H&%I2L#DN4GRA>E*@.2Y12:6-H
MJ0Y+E````````````````````````````````%!%``!,`0$`G,*Z0P``````
M````X``?`0L!!@```@`````````````F`@````(````$`````$````(````"
M```$`````0`!``0```````````0````"``#3DP```P``````$```$``````0
M```0````````$```````````````;`(``%``````````````````````````
M````````````````````````````````````````````````````````````
M```````````````````````````"```@````````````````````````````
M````````;6EX960```!*`0````(````"`````@``````````````````(```
MX`````````````````````#<`@```````#0#````````&@,```H#``#X`@``
M`````'LE<WT*`%6+[(/L%(-E_`"-1>Q0_Q40`D``C47\4(U%[%#_%10"0`#_
M=?QH(`)``/\5"`)``%E9C47\4/\5&`)``&H`_Q4``D``R<.\`@``````````
M``#J`@````(``,P"`````````````"@#```0`@``Q`(`````````````/@,`
M``@"`````````````````````````````-P"````````-`,````````:`P``
M"@,``/@"````````KP!%>&ET4')O8V5S<P!+15).14PS,BYD;&P``-<!4G!C
M4W1R:6YG1G)E94$``.D!575I9%1O4W1R:6YG00#A`55U:61#<F5A=&4``%)0
M0U)4-"YD;&P``)X"<')I;G1F``!-4U9#4E0N9&QL````````````````````
M````````````````````````````````````````````````````````````
M````````````````````````````````````````````````````````````
M````````````````````````````````````````````````````````````
B````````````````````````````````````````````````