@echo off
@if "%OS%"=="Windows_NT" goto :MAIN
@echo This script requires Windows NT 4.0 or later to run properly!
goto :EOF
:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
::
::    This sofware is supplied for instructional purposes only.
::
::    OSR Open Systems Resources, Inc. (OSR) expressly disclaims any warranty
::    for this software.  THIS SOFTWARE IS PROVIDED  "AS IS" WITHOUT WARRANTY
::    OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING, WITHOUT LIMITATION,
::    THE IMPLIED WARRANTIES OF MECHANTABILITY OR FITNESS FOR A PARTICULAR
::    PURPOSE.  THE ENTIRE RISK ARISING FROM THE USE OF THIS SOFTWARE REMAINS
::    WITH YOU.  OSR's entire liability and your exclusive remedy shall not
::    exceed the price paid for this material.  In no event shall OSR or its
::    suppliers be liable for any damages whatsoever (including, without
::    limitation, damages for loss of business profit, business interruption,
::    loss of business information, or any other pecuniary loss) arising out
::    of the use or inability to use this software, even if OSR has been
::    advised of the possibility of such damages.  Because some states/
::    jurisdictions do not allow the exclusion or limitation of liability for
::    consequential or incidental damages, the above limitation may not apply
::    to you.
::
::    OSR Open Systems Resources, Inc.
::    105 Route 101A Suite 19
::    Amherst, NH 03031  (603) 595-6500 FAX: (603) 595-6503
::    email bugs to: bugs@osr.com
::
::
::    MODULE:
::
::        ddkbuild.cmd
::
::    ABSTRACT:
::
::      This file allows drivers to be build with visual studio and visual studio.net
::
::    AUTHOR(S):
::
::        OSR Open Systems Resources, Inc.
:: 
::    REVISION:   V7.0 BETA
::
::      Clean up batch procedure to make it easier to process.
::
::
::    REQUIREMENTS:  Environment variables that must be set.
::
::		NT4BASE - must be set up by user to point to NT4 DDK. (e.g. D:\NT4DDK )
::      W2KBASE - must be set up by user to point to W2K DDK  (e.g D:\Nt50DDK )
::      WXPBASE - must be set up by user to point to WXP DDK  (e.g D:\WINDDK\2600)
::      WNETBASE - must be set up by user to point to WNET DDK (e.g D:\WINDDK\1830)
::
::
::    COMMAND FORMAT:
::
::		ddkbuild -PLATFORM BUILDTYPE DIRECTORY [FLAGS] [-WDF] [-PREFAST]
::
::              PLATFORM is either 
::                   WXP, WXP64, WXP2K - builds using WXP DDK
::                   W2K, W2K64,  - builds using W2k DDK
::                   WNET, WNET64, WNET2K, WNETXP, WNETXP64 - builds using WNET DDK
::                   WNETAMD64  for an AMD64/EM64T WNET build using the WNET DDK
::                   NT4  - build using NT4 DDK (NT4 is the default)
::              BUILDTYPE - free, checked, chk or fre
::				DIRECTORY is the path to the directory to be build.  It can be "."
::              FLAGS - build flags e.g. -cZ etc.   
::              -WDF  - allows the user to perform a Windows Driver Framework build.
::                      this has been tested with the 01.00.5054 version of the 
::                      framework.
::              -PREFAST - performs a prefast build, if prefast is available.
::
::	  BROWSE FILES:
::	
::       This procedure supports the building of BROWSE files to be used by 
::       Visual Studio 6 and by Visual Studio.Net  However, the BSCfiles created
::       by bscmake for the 2 studios are not compatible. When this command procedure
::       runs, it selects the first bscmake.exe found in the path.   So, make
::       sure that the correct bscmake.exe is in the path....  
::
::       Note that if using Visual Studio.NET the .BSC must be added to the project
::       in order for the project to be browsed.
::
::    COMPILERS:
::
::        If you are building NT4 you should really
::        be using the VC 6 compiler.   Later versions of the DDK now contain the
::        compiler and the linker.  This procedure should use the correct compiler.
::
::    GENERAL COMMENTS:
::        This procedure has been cleaned up to be modular and easy to
::		  understand.
::
::		  As of the Server 2003 SP1 DDK ddkbuild now clears the
::        NO_BROWSE_FILE and NO_BINPLACE environment variables so that users
::        can use these features.
::
:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
:: / MAIN function of the script
:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
:MAIN
:: Building "stack frame"
setlocal ENABLEEXTENSIONS & pushd

:: Init some special variables
set OSR_VERSTR=OSR DDKBUILD.BAT V6.5 - OSR, Open Systems Resources, Inc.
set OSR_DEBUG=off

:: Set error messages
set ERR_BadMode=^<build type^> must be 'checked', 'free', 'chk' or 'fre'.
set ERR_NoBASEDIR=NT4BASE, W2KBASE, WXPBASE or WNETBASE environment variable not set. Environment variable must be set by user according to DDK version installed.
set ERR_UnknownBuildType=Unknown type of build. Please recheck parameters.
set ERR_NoDir=The ^<directory^> parameter must be a valid directory.
set ERR_NoTarget=Target directory must contain a SOURCES or DIRS file.
set ERR_NoWdfRoot=WDF_ROOT is not defined, are you using 00.01.5054 or later?

:: Clear the error code variable
set OSR_ERRCODE=0
set prefast_build=0

:: Turn on tracing, use %OSR_TRACE% instead of ECHO
if /i {%OSR_DEBUG%} == {on} (set OSR_TRACE=@echo) else (set OSR_TRACE=rem)

:: Turn on echoing of current line if %OSR_DEBUG% is set to "on"
@echo %OSR_DEBUG%

:: Output version string
@echo %OSR_VERSTR%
%OSR_TRACE% ^(Current module: ^"%~f0^"^)

:: Set the target platform variable
set OSR_TARGET=%1
:: Remove any dashes in the variable
set OSR_TARGET=%OSR_TARGET:-=%

%OSR_TRACE% Now jump to the initialization of commandline and %%BASEDIR%%
:: Calling as a subroutine has 2 advantages:
:: 1. the script does not quit if the label was not found
:: 2. we return to the line after the call and can check variables there
call :%OSR_TARGET%Build

:: Check for existing %BASEDIR%
if {%BASEDIR%}=={} call :ErrorWithUsage 1 "%ERR_NoBASEDIR%" & goto :END
set PATH=%BASEDIR%\bin;%PATH%

%OSR_TRACE% We returned from the variable initialization
if not DEFINED OSR_BUILDNAME call :ErrorWithUsage 1 "%ERR_UnknownBuildType%" & goto :END
if not DEFINED OSR_CMDLINE call :ErrorWithUsage 1 "%ERR_UnknownBuildType%" & goto :END

%OSR_TRACE% Hurrah, all the variables have been initialized, continuing
:: Proceed with common build steps
goto :CommonBuild

:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
:: Initialize variables specific to the respective platform
:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

:: NT 4.0 build using NT4 DDK
:NT4Build
set OSR_BUILDNAME=NT4 BUILD using NT4 DDK
set BASEDIR=%NT4BASE%
set OSR_CMDLINE=%%BASEDIR%%\bin\setenv.bat %%BASEDIR%% %%mode%% "%%MSDEVDIR%%"
goto :EOF

:: W2K build for 32bit using WNET DDK
:WNET2KBuild
set OSR_BUILDNAME=W2K BUILD using WNET DDK
set BASEDIR=%WNETBASE%
set OSR_CMDLINE=%%BASEDIR%%\bin\setenv.bat %%BASEDIR%% W2K %%mode%%
goto :EOF

:: WXP build for 32bit using WNET DDK
:WNETXPBuild
set OSR_BUILDNAME=WXP BUILD using WNET DDK
set BASEDIR=%WNETBASE%
set OSR_CMDLINE=%%BASEDIR%%\bin\setenv.bat %%BASEDIR%% %%mode%% WXP
goto :EOF

:: WXP build for 64bit using WNET DDK
:WNETXP64Build
set OSR_BUILDNAME=WXP 64 BIT BUILD using WNET DDK
set BASEDIR=%WNETBASE%
set OSR_CMDLINE=%%BASEDIR%%\bin\setenv.bat %%BASEDIR%% %%mode%% 64 WXP
goto :EOF

:: WNET build for 64bit (Intel) using WNET DDK
:WNET64Build
set OSR_BUILDNAME=WNET 64 BIT BUILD using WNET DDK
set BASEDIR=%WNETBASE%
set OSR_CMDLINE=%%BASEDIR%%\bin\setenv.bat %%BASEDIR%% %%mode%% 64 WNET
goto :EOF

:: WNET build for 64bit (AMD) using WNET DDK
:WNETAMD64Build
set OSR_BUILDNAME=WNET 64 BIT BUILD using WNET DDK
set BASEDIR=%WNETBASE%
set OSR_CMDLINE=%%BASEDIR%%\bin\setenv.bat %%BASEDIR%% %%mode%% AMD64 WNET
goto :EOF

:: WNET build for 32bit using WNET DDK
:WNETBuild
set OSR_BUILDNAME=WNET 32 BIT BUILD using WNET DDK
set BASEDIR=%WNETBASE%
set OSR_CMDLINE=%%BASEDIR%%\bin\setenv.bat %%BASEDIR%% %%mode%%
goto :EOF

:: WXP build for 64bit (Intel) using WXP DDK
:WXP64Build
set OSR_BUILDNAME=WXP 64 BIT BUILD using WXP DDK
set BASEDIR=%WXPBASE%
set OSR_CMDLINE=%%BASEDIR%%\bin\setenv.bat %%BASEDIR%% %%mode%% 64
goto :EOF

:: WXP build for 32bit using WXP DDK
:WXPBuild
set OSR_BUILDNAME=WXP 32 BIT BUILD using WXP DDK
set BASEDIR=%WXPBASE%
set OSR_CMDLINE=%%BASEDIR%%\bin\setenv.bat %%BASEDIR%% %%mode%%
goto :EOF

:: W2K build for 32bit using WXP DDK
:WXP2KBuild
set OSR_BUILDNAME=W2K 32 BIT BUILD using WXP DDK
set BASEDIR=%WXPBASE%
set OSR_CMDLINE=%%BASEDIR%%\bin\w2k\set2k.bat %%BASEDIR%% %%mode%%
goto :EOF

:: W2K build for 64bit (Intel) using W2K DDK
:W2K64Build
set OSR_BUILDNAME=W2K 64 BIT BUILD using W2K DDK
set BASEDIR=%W2KBASE%
set OSR_CMDLINE=%%BASEDIR%%\bin\setenv64.bat %%BASEDIR%% %%mode%%
goto :EOF

:: W2K build for 32bit using W2K DDK
:W2KBuild
set OSR_BUILDNAME=W2K 32 BIT BUILD using W2K DDK
set BASEDIR=%W2KBASE%
set OSR_CMDLINE=%%BASEDIR%%\bin\setenv.bat %%BASEDIR%% %%mode%%
goto :EOF

:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
:: All builds go here for the rest of the procedure.  Now,
:: we are getting ready to call build.  The big problem
:: here is to figure our the name of the buildxxx files
:: being generated for the different platforms.
:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

:CommonBuild
:: Remove first command line arg
shift
call :SetMode %1
if not {%OSR_ERRCODE%} == {0} call :ErrorWithUsage 1 "%ERR_BadMode%" & goto :END

call :CheckTargets %2
if {%OSR_ERRCODE%} == {2} call :ErrorWithUsage %OSR_ERRCODE% "%ERR_NoTarget%" & goto :END
if not {%OSR_ERRCODE%} == {0} call :ErrorWithUsage %OSR_ERRCODE% "%ERR_NoDir%" & goto :END

:: Resolve any variables in the command line string
set OSR_CMDLINE=%OSR_CMDLINE%

pushd .
:: This external script prepares the build environment (e.g. setenv.bat)
call %OSR_CMDLINE%
popd

:: ----------------------------------------------------------------------------
:: Setting global variables for the scope of this CMD session
set NO_BROWSER_FILE=
set NO_BINPLACE=

:: Set additional variables at this point or do whatever you please
if exist %buildDirectory%\ddkprebld.cmd @(
  echo Performing pre-build steps ...
  call %buildDirectory%\ddkprebld.cmd
)

set mpFlag=-M
if {%BUILD_ALT_DIR%}=={} goto :NT4

:: W2K sets this!
set W2kEXT=%BUILD_ALT_DIR%
set mpFlag=-MI

:NT4
if {%NUMBER_OF_PROCESSORS%}=={} set mpFlag=
if {%NUMBER_OF_PROCESSORS%}=={1} set mpFlag=

:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
:: Determine the settings of flags, WDF and PREFAST in
:: other words what was set for %3 and beyond....
:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
@echo build in directory %2 with arguments %3 %4 %5 (basedir %BASEDIR%)

cd /D %~s2
set bflags=-Ze
set bscFlags=""

set buildDirectory=%2

:ContinueParsing
if {%3} == {} goto :DONE
if {%3} == {/a} goto :RebuildallFound
if /i {%3} == {-WDF} goto :WDFFound
if /i {%3} == {-PREFAST} goto :PrefastFound
set bscFlags=/n
set bflags=%3 -e
:: Remove first arg
shift
goto :ContinueParsing

:WDFFound
shift
if "%WDF_ROOT%" == "" goto :ErrNoWdfRoot
pushd .
call %WDF_ROOT%\set_wdf_env.cmd
popd
set DEBUG=on
goto :ContinueParsing

:PrefastFound
shift
set prefast_build=1
goto :ContinueParsing

:RebuildallFound
shift
set bscFlags=/n
set bflags=-cfeZ
goto :ContinueParsing

:DONE

if exist build%W2kEXT%.err   erase build%W2kEXT%.err
if exist build%W2kEXT%.wrn   erase build%W2kEXT%.wrn
if exist build%W2kEXT%.log   erase build%W2kEXT%.log
if exist prefast%W2kEXT%.log erase prefast%W2kEXT%.log

if not {%prefast_build%} == {0} goto RunPrefastBuild
@echo run build %bflags% %mpFlag% for %mode% version in %buildDirectory%
pushd .
build  %bflags% %mpFlag%
goto BuildComplete

:RunPrefastBuild
@echo run prefast build %bflags% %mpFlag% for %mode% version in %buildDirectory%
pushd .
prefast build  %bflags% %mpFlag%
prefast list > prefast%W2kEXT%.log
goto :BuildComplete

:BuildComplete
if {%errorlevel%} GTR {0} set OSR_ERRCODE=%errorlevel%
popd

@echo %OSR_DEBUG%

:: assume that the onscreen errors are complete!

@echo =============== build warnings ======================
if exist build%W2kEXT%.wrn findstr "warning[^.][DRCLU][0-9]*" build%W2kEXT%.log
if exist build%W2kEXT%.log findstr "error[^.][DRCLU][0-9]*" build%W2kEXT%.log

if {%prefast_build%} == {0} goto :SkipPrefastWarnings
@echo =============== prefast warnings ======================
if exist prefast%W2kEXT%.log findstr "warning[^.][CLU]*" prefast%W2kEXT%.log
:SkipPrefastWarnings
@echo.
@echo.
@echo Build complete
@echo Building browse information files
if exist buildbrowse.cmd goto :doBrowsescript
set sbrlist=sbrList.txt
if not exist sbrList%CPU%.txt goto :sbrDefault
set sbrlist=sbrList%CPU%.txt

:sbrDefault
if not exist %sbrlist% goto :postBuildSteps
if %bscFlags% == "" goto :noBscFlags
bscmake %bscFlags% @%sbrlist%
goto :postBuildSteps

:noBscFlags
bscmake @%sbrlist%
goto :postBuildSteps

:doBrowsescript
call buildBrowse %mode%
goto :postBuildSteps

:: Perform whatever post-build steps
:postBuildSteps
if exist %buildDirectory%\ddkpostbld.cmd @(
  echo Performing post build steps ...
  call %buildDirectory%\ddkpostbld.cmd
)
goto :END
:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
:: \ MAIN function of the script
:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
::  / SetMode
::    Subroutine to validate the mode of the build passed in. It must be free,
::    FREE, fre, FRE or checked, CHECKED, chk, CHK. Anything else is an error.
:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
:SetMode
set mode=
for %%f in (free FREE fre FRE) do if %%f == %1 set mode=free
for %%f in (checked CHECKED chk CHK) do if %%f == %1 set mode=checked
if "%mode%" =="" set OSR_ERRCODE=1
goto :EOF
:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
::  \ SetMode
:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
:: / CheckTargets subroutine
::   Subroutine to validate that the target directory exists and that there is
::   either a DIRS or SOURCES and MakeFile in it.
:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
:CheckTargets
:: Building "stack frame"
setlocal & pushd & set OSR_ERRCODE=0
if "%1" NEQ "" goto :CheckTargets1
set OSR_ERRCODE=1
goto :CheckTargets_Return
:CheckTargets1
if exist %1 goto :CheckTargets2
set OSR_ERRCODE=1
goto :CheckTargets_Return
:CheckTargets2
if not exist %1\DIRS  goto :CheckTargets3
set OSR_ERRCODE=0
goto :CheckTargets_Return
:CheckTargets3
if exist %1\SOURCES  goto :CheckTargets4
set OSR_ERRCODE=2
goto :CheckTargets_Return
:CheckTargets4
if exist %1\MAKEFILE  goto :CheckTargets5
set OSR_ERRCODE=2
goto :CheckTargets_Return
:CheckTargets5
set OSR_ERRCODE=0
:CheckTargets_Return
:: Cleaning "stack frame" and returning error code into global scope
popd & endlocal & set OSR_ERRCODE=%OSR_ERRCODE%
goto :EOF
:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
:: \ CheckTargets subroutine
:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
:: / ErrorWithUsage subroutine
::   This one will take the passed in parameters and build a nice error
::   message which is returned to the user along with the usage hints.
::
:: NEVER RETURNS! IT JUST FALLS THROUGH INTO ":USAGE"
:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
:ErrorWithUsage
@set OSR_ERRCODE=%1
@set OSR_ERRMSG=%2
@set OSR_ERRMSG=%OSR_ERRMSG:"=%
@set OSR_ERRMSG=%OSR_ERRMSG:'="%
@set OSR_ERRMSG=ERROR: %OSR_ERRMSG%
@echo.
@echo %OSR_ERRMSG%
:: Never return from ErrorWithUsage
:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
:: \ ErrorWithUsage subroutine
:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
:: Usage output
:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
:USAGE
@echo.
@echo.
@echo Usage:
@echo ddkbuild ^<platform^> ^<build type^> ^<directory^> [flags] [-WDF] [-PREFAST]
@echo.
@echo Platform values:
@echo       -W2K       to indicate W2K        build using %%W2KBASE%%
@echo       -W2K64     to indicate W2K  IA64  build using %%W2KBASE%%
@echo       -WXP       to indicate WXP        build using %%WXPBASE%%
@echo       -WXP64     to indicate WXP  IA64  build using %%WXPBASE%%
@echo       -WXP2K     to indicate W2K        build using %%WXPBASE%%
@echo       -WNET      to indicate WNET       build using %%WNETBASE%%
@echo       -WNET64    to indicate WNET IA64  build using %%WNETBASE%%
@echo       -WNETXP    to indicate WXP        build using %%WNETBASE%%
@echo       -WNETXP64  to indicate WXP  IA64  build using %%WNETBASE%%
@echo       -WNETAMD64 to indicate WNET AMD64 build using %%WNETBASE%%
@echo       -WNET2K    to indicate W2K        build using %%WNETBASE%%
@echo       -NT4       to indicate NT4        build using %%NT4BASE%%
@echo.
@echo Build types:
@echo        checked
@echo        chk       indicates a checked build
@echo        free
@echo        fre       indicates a free build
@echo.
@echo Remaining parameters:
@echo        directory path to build directory, try . (cwd)
@echo        flags     any random flags you think should be passed to build (try /a
@echo                  for clean)
@echo       -WDF       performs a WDF build
@echo       -PREFAST   performs a PREFAST build
@echo.
@echo Examples:
@echo       ^"ddkbuild -NT4 checked .^" (for NT4 BUILD)
@echo       ^"ddkbuild -WXP64 chk .^"
@echo       ^"ddkbuild -WXP chk c:\projects\myproject^"
@echo       ^"ddkbuild -WNET64 chk .^"      (IA64 bit build)
@echo       ^"ddkbuild -WNETAMD64 chk .^"   (AMD64/EM64T bit build)
@echo       ^"ddkbuild -WNETXP chk . -cZ -WDF^"
@echo       ^"ddkbuild -WNETXP chk . -cZ -PREFAST^"
@echo.
@echo       In order for this procedure to work correctly for each platform, it
@echo       requires an environment variable to be set up for certain platforms.
@echo       The environment variables are as follows:
@echo.
@echo       %%NT4BASE%%  - Set this up for ^"-NT4^" builds
@echo       %%W2KBASE%%  - Set this up for ^"-W2K^" and ^"-W2K64^" builds
@echo       %%WXPBASE%%  - Set this up for ^"-WXP^", ^"-WXP64^", ^"-WXP2K^" builds
@echo       %%WNETBASE%% - Set this up for ^"-WNET^", ^"-WNET64^", ^"-WNETXP^", ^"-WNETXP64^",
@echo                    ^"-WNETAMD64^", and ^"-WNET2K^" builds
@echo.
@echo       %%WDF_ROOT%% must be set if attempting to do a WDF Build.
@echo.
@echo.
@echo   %OSR_VERSTR%
@echo     report any problems found to info@osr.com
@echo.

:END
popd & endlocal
exit /b %OSR_ERRCODE%
@echo ddkbuild complete
